% This is the main file to visualize and evaluate the performance of Jacly
% and compare it with GGM. You can run this code directly after the optimization by
% Jacly is finished. Please do not clear the workspace after the
% optimization and before running this code, otherwise you have to load
% some of the inference results such as "FinalResult" into the workspace.

%%%%%% All points on the plot %%%%%%%%%%%%%%%%%%%%%%%
R=FinalResult.Inference;
f=cell2mat(R(2:end,2));
residual=cell2mat(R(2:end,3));
N0=cell2mat(R(2:end,4));
TPR=cell2mat(R(2:end,5));
FPR=cell2mat(R(2:end,6));

NN = N0./10;
Reslog = -log10(residual);

figure;
plot(NN,Reslog,'bs')
fprintf('Please select a point as the elite Jacobian (based on both residual and number of zeros)\n');
gname

SPindex = input('Index of the point selected by screening? ');
spN0 = N0(SPindex);
spRes = -log10(residual(SPindex));
N0interval = 0.05;
Resinterval = 0.05;

if isequal(InputType,'SDE data')
    sdeData = SDEdata;
    n = size(sdeData,2);
else
    n = size(C,1);
end

N0_lb = round((1-N0interval)*spN0);
N0_ub = round((1+N0interval)*spN0);
min_res = (1-Resinterval)*spRes;

figure;
bestfoundH = plot(N0./10,-log10(residual),'bs');
title('Best-Found Jacobians')
xlabel('Number of zeros / 10')
ylabel('-log10(residual)')
caxh = gca;
ylineH1 = line([N0_lb/10 N0_lb/10],get(caxh,'YLim'),'Color',[1 0 0]);
ylineH2 = line([N0_ub/10 N0_ub/10],get(caxh,'YLim'),'Color',[1 0 0]);
xlineH = line(get(caxh,'XLim'),[min_res min_res],'Color',[1 0 0]);
hold on;
eliteH = plot(caxh,[spN0./10],[spRes],'*r','markersize',12);
legend([bestfoundH eliteH],'Best-Found Jacobians','Elite Jacobian');
pause
freethreshold = 0.5;
ggmthreshold = 0;

N0range = find(N0<=N0_ub);
N0range = N0range(find(N0(N0range)>=N0_lb));
indx = N0range;
selectedpoints = indx(find(-log10(residual(indx))>=min_res));
%selectedpoints = SPindex; % active when you want to work only on the best point and not the gruop of points
finalList = [f(selectedpoints) -log10(residual(selectedpoints)) N0(selectedpoints) TPR(selectedpoints) FPR(selectedpoints)];
finalList = unique(finalList,'rows');
Nselpoints = size(finalList,1);

bpJacs = FinalResult.BestJacs(:,:,selectedpoints);
bpJacVec = [];
for i=1:size(bpJacs,3)
    bpjac = bpJacs(:,:,i);
    % pruning
    %bpjac = pruneTriangles(bpjac,0.3,1);
    % pruning
    bpJacVec(:,i) = bpjac(:);
end
bpJacVec = bpJacVec';
bpJacVec = unique(bpJacVec,'rows');
Nseljacs = size(bpJacVec,1);
if Nselpoints~=Nseljacs
    error('There is a problem in selection of points or Jacobians!')
end
fprintf('Number of unique points in the bounded area: %i\n',Nselpoints)
avgfinalList = mean(finalList,1);
avgres = avgfinalList(2);
avgtpr = avgfinalList(4);
avgfpr = avgfinalList(5);
fprintf('Average residual = %d\n',avgres)
fprintf('Average TPR = %d\n',avgtpr)
fprintf('Average FPR = %d\n',avgfpr)

binbpJacVec = bpJacVec;
binbpJacVec(binbpJacVec~=0) = 1;


rJ = This_J;
rJvec = rJ(:);
binrJvec = rJvec;
binrJvec(binrJvec~=0) = 1;

if isequal(InputType,'SDE data')
%     ggmnet = GGMinfer(sdeData,0.01);
%     I = triu(ones(n),1);
%     ggmJac = zeros(n);
%     ggmJvec = ggmJac(:);
%     ggmJvec(find(I)) = ggmnet;
%     ggmJ = reshape(ggmJvec,n,n);
%     ggmJ = ggmJ + ggmJ' + eye(n);
    [Rho, pval] = partialcorr(sdeData);
    ggmJ = pval;
    ggmJ(ggmJ < 0.01) = 1;
    ggmJ(ggmJ ~= 1) = 0;
    ggmJvec = ggmJ(:);
end

binjacs = binbpJacVec;
diagindx = find(eye(n));
binjacs(:,diagindx) = [];
binrJvec(diagindx) = [];
if isequal(InputType,'SDE data')
    ggmJvec(diagindx) = [];
end
%hmData(:,diagindx) = [];

Sbinjacs = sum(binjacs,1);
PSbinjacs = Sbinjacs./size(binjacs,1);

PSbinjacs(PSbinjacs<freethreshold) = 0;
if isequal(InputType,'SDE data')
    for i=1:length(PSbinjacs)
        if (PSbinjacs(i)<ggmthreshold && ggmJvec(i)==0)
            PSbinjacs(i) = 0;
        end
    end
end

binPSbinjacs = PSbinjacs;
binPSbinjacs(binPSbinjacs~=0) = 1;

base = binrJvec;
sample = binPSbinjacs';

T=find(sample==base);
TP=length(find(sample(T)==1));
TN=length(find(sample(T)==0));

F=find(sample~=base);
FP=length(find(sample(F)==1));
FN=length(find(sample(F)==0));

tpr=TP/length(find(base==1));
fpr=FP/length(find(base==0));

JacLySparsity = (length(sample)-sum(sample))/length(sample);
TrueSparsity = (length(base)-sum(base))/length(base);

fprintf('\nEstimated Directed Network by JacLy\n')
fprintf('TPR: %d\nFPR: %d\n',tpr,fpr)
fprintf('sparsity: %1.2f\n',JacLySparsity);
fprintf('TrueSparsity: %1.2f\n',TrueSparsity);

if isequal(InputType,'SDE data')
    base = binrJvec;
    sample = ggmJvec;
    
    T=find(sample==base);
    TP=length(find(sample(T)==1));
    TN=length(find(sample(T)==0));
    
    F=find(sample~=base);
    FP=length(find(sample(F)==1));
    FN=length(find(sample(F)==0));
    
    tpr=TP/length(find(base==1));
    fpr=FP/length(find(base==0));
    
    ggmSparsity = (length(sample)-sum(sample))/length(sample);
    
    fprintf('\nEstimated Undirected Network by GGM\n')
    fprintf('TPR: %d\nFPR: %d\n',tpr,fpr)
    fprintf('sparsity: %1.2f\n',ggmSparsity);
end

% hmData = [binPSbinjacs;binrJvec';binjacs;ggmJvec'];
% cg = clustergram(hmData,'cluster','column','RowPDist','cityblock');

%%%%%%%%%%%%%%%%%%%%%%% END %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




