% The Main Script
% This is the main file to run Jacly for a network inference problem

clear;
clc;
% warning off all

%%%%%%%%% Setting the inputs %%%%%%%%%% START
rep = 10;    % Number of times to repeat GA for the same problem (same scaling factor value and etc) because of stochastic nature of GA
tic;

% The Reference Jacobian: This is for the cases when you already have the
% real Jacobian of your system and you want to validate the inference
% results
Modelname = 'Yeast'; % It is the name of the .mat file that contains the corresponding jacobian under the name of This_J
load(Modelname)
realJ = This_J;
[n,m] = size(realJ);
if n~=m || isempty(realJ) || n==1
    error('The given Jacobian is not a squar or valid matrix!')
end

% Parameters to be set according to our expectations from network structure
MinSparsity = 0.5; % How much sparse you expect the network to be. 1: Fully connected, 0: Not even a single edge in the net
MinSparsity = MinSparsity*(n/(n-1)); % Cancelling the effect of autoregulations in jacobian

% Input Type: "SDE data" or "Exact covariance"
InputType = 'SDE data';

%%%% SDE data: Activate this section if the input is SDE data and comment the "Exact covariance" section 
load yeastSDE % load corresponding file for the SDE data

    %%%% Selecting a random subset of data: Activate this section if you want to select a lower number of data replicates as input to the algorithm
    %     SDEdata = SDEdata(randperm(size(SDEdata,1)),:);
    %     SDEdata = SDEdata(1:100,:);
    %%%%%%%%%%%%%

    %%%% Data Normalization: Activate if you want to use normalized data as input to the algorithm
    %     SDEdata = SDEdata./repmat(max(abs(SDEdata)),size(SDEdata,1),1);
    %%%%%%%%%%%%%

sdeDi = std(SDEdata);
D = diag(sdeDi); % The fluctuation matrix
d = -2*D(:); % The fluctuation vector
C = cov(SDEdata); % The covariance matrix
%%%%%%%%%%%%% SDE data: END

%%%% Exact covariance: Activate this section if the input is exact covariance and comment the "SDE data" section 
%     Di = 5e-3;  % The fluctuation parameter
%     D = diag(Di*ones(n,1)); % The fluctuation matrix
%     d = -2*D(:); % The fluctuation vector
%     C = lyap2(realJ,2*D);
%%%%%%%%%%%%% Exact covariance: END



% Adding noise or no??
Noise = 'No'; % 'Yes' if you want to add noise to covariance, 'No' if you dont want.
if isequal(Noise,'Yes')
    C = addnoise(C,1,0.005);
end


A = MakeAoutofC(C);    % This A multiplied by vec(J') is equivalent to A*j of equation 1

residulimit = 1e-5;

%%%%%%%%% Setting the inputs %%%%%%%%%% END


%%%%%%%% Constraints Offered by CORrelation (CObyCOR) %%%%%%%% START
Correlation_Type = 'None'; % it can be 'PC' or 'GGM' or 'None'
if isequal(Correlation_Type,'None')
    links = [];
    nolinks = [];
else
    [links,nolinks,linkscontrol,nolinkscontrol] = CObyCOR(Correlation_Type,C,realJ);
end
%%%%%%%% Constraints Offered by CORrelation (CObyCOR) %%%%%%%% END

% Priory knowledge on some True Negatives
knownTNs.rowindices = []; %[11.*ones(1,12) 14.*ones(1,11)];
knownTNs.colindices = []; %[3 5:9 12 13 15:18 1 2 4 6:12 18];

% Symmetric TNs: making the known TNs symmetric
rowids = [knownTNs.rowindices knownTNs.colindices];
colids = [knownTNs.colindices knownTNs.rowindices];

TNids = sub2ind(size(realJ),rowids,colids);
ww = ones(size(realJ));
ww(TNids) = 0;
I = triu(ones(size(ww)),1);
wwupvec = ww(find(I));
TNupperIndices = find(wwupvec==0);


%%%%%%%% Genetic Algorithm Part %%%%%%%%%%%%% START

indboyut = n*(n-1)/2 - (length(links)+length(nolinks)+length(TNids)/2);   % individual length

minsparsity = (MinSparsity*(n*(n-1)/2) - length(nolinks) - (length(TNids)/2))/indboyut; % minsparsity for each individual

% The Genetic Algorithm parameters
load yeastSDEinitpop % corresponding file for initial population generated based on a primary correlation
initpop(:,TNupperIndices) = [];

Sparsecreation       = @(GenomeLength,FitnessFcn,options) sparsecreation(GenomeLength,FitnessFcn,options,minsparsity);
Sparsechildcrossover = @(parents,options,GenomeLength,FitnessFcn,unused,thisPopulation) sparsechildcrossover(parents,options,GenomeLength,FitnessFcn,unused,thisPopulation,minsparsity);
Sparsemakermutation  = @sparsemakermutation;

GAoptions.PopulationType        = 'bitstring';
GAoptions.PopInitRange          = [];
GAoptions.PopulationSize        = [50 50 50];
GAoptions.EliteCount            = 3;
GAoptions.CrossoverFraction     = [];
GAoptions.ParetoFraction        = [];
GAoptions.MigrationDirection    = [];
GAoptions.MigrationInterval     = [];
GAoptions.MigrationFraction     = [];
GAoptions.Generations           = 800;
GAoptions.TimeLimit             = [];
GAoptions.FitnessLimit          = [];
GAoptions.StallGenLimit         = 800;
GAoptions.StallTimeLimit        = [];
GAoptions.TolFun                = [];
GAoptions.TolCon                = [];
GAoptions.InitialPopulation     = initpop;
GAoptions.InitialScores         = [];
GAoptions.InitialPenalty        = [];
GAoptions.PenaltyFactor         = [];
GAoptions.PlotInterval          = [];
GAoptions.CreationFcn           = Sparsecreation;
GAoptions.FitnessScalingFcn     = @fitscalingtop;
GAoptions.SelectionFcn          = @selectionuniform;
GAoptions.CrossoverFcn          = Sparsechildcrossover;
GAoptions.MutationFcn           = {[Sparsemakermutation]  [0.05]};
GAoptions.DistanceMeasureFcn    = [];
GAoptions.HybridFcn             = [];
GAoptions.Display               = 'off';
GAoptions.PlotFcns              = []; %{[@gaplotbestf]};
GAoptions.OutputFcns            = [];
GAoptions.Vectorized            = 'on';
GAoptions.UseParallel           = 'always';

% Scanning the scaling factor lambda 
initial_lambda = 0.01;
final_lambda = 0.1;
N_lambda = 20;
Lambda_values = linspace(initial_lambda,final_lambda,N_lambda);
for i=1:N_lambda
    LambdaList(rep*(i-1)+1:rep*i,1) = Lambda_values(i);
end

for gg = 1:length(Lambda_values)
    lambda = Lambda_values(gg);
    fprintf('\n Lambda %i = %d\n',gg,lambda)
    % Other inputs to the fitness function
    OtherInputs.n = n;
    OtherInputs.lambda = lambda;
    OtherInputs.A = A;
    OtherInputs.d = d;
    OtherInputs.residulimit = residulimit;
    OtherInputs.links = links;
    OtherInputs.nolinks = nolinks;
    OtherInputs.TNids = TNids;
    OtherInputs.LambdaList = LambdaList;
    
    if isequal(GAoptions.Vectorized,'on')
        FitnessFun = @(indboyut) anonymous_vectorized_fitness(indboyut,OtherInputs);
    else
        FitnessFun = @(indboyut) anonymous_fitness(indboyut,OtherInputs);
    end
    
    % Optimization takes place in the following loop
    for i = 1:rep
        fprintf('GA run: %i\n',i)
        [x,fval,exitflag,output,population,scores]=ga(FitnessFun,indboyut,[],[],[],[],[],[],[],GAoptions);
        X(:,rep*(gg-1)+i) = x';    % Collection of best bitstring found at each round of optimization
    end
    %%%%%%%% Genetic Algorithm Part %%%%%%%%%%%%% END
end
toc;

FinalResult = inference(X,realJ,OtherInputs);   % A data structure containing the results

save([Modelname '_ScanResults']);

AnalyzeTheScanResults;




