
function f = anonymous_vectorized_fitness(Jmatrix,OtherInputs)
% This is the objective function for Genetic Algorithm. In this function,
% an objective value is being calculated based on both number of zeros in
% the estimated Jacobian and also the residual of the Lyapunov equation for
% that Jacobian.

% warning off all
[nofpop,nofvar] = size(Jmatrix);
Jmatrix = Jmatrix'; % setting each column to be one individual

n = OtherInputs.n;
lambda = OtherInputs.lambda;
A = OtherInputs.A;
d = OtherInputs.d;
residulimit = OtherInputs.residulimit;
links = OtherInputs.links;
nolinks = OtherInputs.nolinks;
TNids = OtherInputs.TNids;

virJ = 2*ones(n);   % Basis for Virtual Jacobian
virJ = virJ - eye(n);   % main diagonal set to 1
virJvec = virJ(:);     % vectorized virJ
virJvec(links) = 1;
virJvec(nolinks) = 0;
virJvec(TNids) = 0;

B = reshape(virJvec,n,n);
I = triu(ones(size(B)),1);
Bupvec = B(find(I));
Bupmat = repmat(Bupvec,1,nofpop);
Bupmat(find(Bupmat==2)) = Jmatrix; % assigning the bitstring elements to the Virtual Jacobian

for k = 1:nofpop
    virJ = zeros(n);
    virJvec = virJ(:);
    virJvec(find(I)) = Bupmat(:,k);
    virJ = reshape(virJvec,n,n);
    virJ = virJ + virJ' + eye(n);   % Final Virtual Binary Jacobian
    
    virJvec = virJ(:);
    
    zeroindx = find(virJvec==0);
    
    B = A;
    B(:,zeroindx) = [];
    xopt = B\d;
    ress = (norm(B*xopt-d))^2;
    
    reskislim = 1e-5;   % We decided to make it a constant instead of being dependent on residulimit
    nzero1 = length(find(abs(xopt)<= reskislim )) + length(zeroindx);
    nzeromax = ceil(0.9*(n^2-n));
    nzero = min(nzero1,nzeromax);
    
    if ress <= residulimit
        mini=-log10(residulimit)-((-log10(residulimit))+log10(ress))/20;
    else
        mini=-log10(ress);
    end
    
    f(k) = -(mini+lambda*nzero);
end


