
function [OUTPUT] = inference(X,realJ,OtherInputs)
% This function takes in the estimated Jacobians as output of optimization
% and compare them to the real Jacobian to calculate TPR, FPR and make a
% table of the inference result.

n = OtherInputs.n;
lambda = OtherInputs.lambda;
A = OtherInputs.A;
d = OtherInputs.d;
residulimit = OtherInputs.residulimit;
links = OtherInputs.links;
nolinks = OtherInputs.nolinks;
TNids = OtherInputs.TNids;
LambdaList = OtherInputs.LambdaList;

InferenceResult = cell(size(X,2)+1,13);
InferenceResult(1,:) = {'i','fitness','Residual','# of 0s','TPR(D)','FPR(D)','TPR(Un)','FPR(Un)','TPR(WIR)','FPR(WIR)','Pearson','Spearman','lambda'};

for ii = 1:size(X,2)
    x = X(:,ii);
    
    virJ = 2*ones(n);   % Basis for Virtual Jacobian
    virJ = virJ - eye(n);   % main diagonal set to 1
    virJvec = virJ(:);     % vectorized virJ
    virJvec(links) = 1;
    virJvec(nolinks) = 0;
    virJvec(TNids) = 0;
    
    B = reshape(virJvec,n,n);
    I = triu(ones(size(B)),1);
    Bupvec = B(find(I));
    Bupvec(find(Bupvec==2)) = x; % assigning the bitstring elements to the Virtual Jacobian
    
    virJ = zeros(n);
    virJvec = virJ(:);
    virJvec(find(I)) = Bupvec;
    virJ = reshape(virJvec,n,n);
    virJ = virJ + virJ' + eye(n);   % Final Virtual Binary Jacobian
    
    virJvec = virJ(:);
    
    zeroindx = find(virJvec==0);
        
    B = A;
    B(:,zeroindx) = [];
    xopt = B\d;
    ress = (norm(B*xopt-d))^2;
        
    reskislim = 1e-5;   % We decided to make it a constant instead of being dependent on residulimit
    xopt(abs(xopt)<=reskislim) = 0;
    virjt = ones(n*n,1);
    virjt(zeroindx) = 0;
    virjt(find(virjt)) = xopt;
        
    nzero = length(find(virjt==0));
    Residual = ress;
    
    virJt = reshape(virjt,n,n);
    virJ  = virJt';
    
    if ress <= residulimit
        mini=-log10(residulimit)-((-log10(residulimit))+log10(ress))/20;
    else
        mini=-log10(ress);
    end
    
    f = -(mini+lambda*nzero);
    
    Jt = realJ';
    jt = Jt(:);
    jt(jt~=0) = 1;  % changing the vector of transpose of the real J to a binary one
    
    virjt(virjt~=0) = 1;    % Also making it binary for the result of optimization
    
    %%%%%%%%% Inference for Directed Network %%%%%%% START
    base = jt;  % Basic element of Comparison
    sample = virjt; % Sample element of Comparison
    
    %%%% Update 5 July 2015
    %%%%%% Removal of main diagnol from comparisons
    diagindx = find(eye(n));
    base(diagindx) = [];
    sample(diagindx) = [];
    %%%%
    
    T=find(sample==base);
    TP=length(find(sample(T)==1));
    TN=length(find(sample(T)==0));
    
    F=find(sample~=base);
    FP=length(find(sample(F)==1));
    FN=length(find(sample(F)==0));
    
    tpr1=TP/length(find(base==1));
    fpr1=FP/length(find(base==0));
    %%%%%%%%% Inference for Directed Network %%%%%%% END
    
    %%%%%%%%% Inference for Undirected Network %%%%%%% START
    base = jt;  % Basic element of Comparison
    sample = virjt; % Sample element of Comparison
    Base = reshape(base,n,n);
    Sample = reshape(sample,n,n);
    
    % Making Base and Sample symmetric. Without using a loop :)
    [row,col] = find(Base==1);
    lin_indx = sub2ind(size(Base),row,col);
    Base = Base';
    Base(lin_indx) = 1;
    [row,col] = find(Sample==1);
    lin_indx = sub2ind(size(Sample),row,col);
    Sample = Sample';
    Sample(lin_indx) = 1;
    
    base = Base(:);
    sample = Sample(:);
    
    %%%% Update 5 July 2015
    %%%%%% Removal of main diagnol from comparisons
    diagindx = find(eye(n));
    base(diagindx) = [];
    sample(diagindx) = [];
    %%%%
    
    T=find(sample==base);
    TP=length(find(sample(T)==1));
    TN=length(find(sample(T)==0));
    
    F=find(sample~=base);
    FP=length(find(sample(F)==1));
    FN=length(find(sample(F)==0));
    
    tpr2=TP/length(find(base==1));
    fpr2=FP/length(find(base==0));
    %%%%%%%%% Inference for Undirected Network %%%%%%% END
    
    %%%%%%%%% Inference for Directed Network after elemination of %%%%%%% START
    %%%%%%%%       Weak Interactions in the Real Jacobian          %%%%%%
    
    WIL = 1; % values less than WIL (Weak Interaction Limit) are set to 0 in real Jacobian
    modifiedJ = realJ;
    modifiedJt = modifiedJ';
    modjt = modifiedJt(:);
    modjt(abs(modjt)<WIL) = 0;    % elimination of weak interactions
    modjt(modjt~=0) = 1;
    
    base = modjt;  % Basic element of Comparison
    sample = virjt; % Sample element of Comparison
    
    %%%% Update 5 July 2015
    %%%%%% Removal of main diagnol from comparisons
    diagindx = find(eye(n));
    base(diagindx) = [];
    sample(diagindx) = [];
    %%%%
    
    T=find(sample==base);
    TP=length(find(sample(T)==1));
    TN=length(find(sample(T)==0));
    
    F=find(sample~=base);
    FP=length(find(sample(F)==1));
    FN=length(find(sample(F)==0));
    
    tpr3=TP/length(find(base==1));
    fpr3=FP/length(find(base==0));
    %%%%%%%%% Inference for Directed Network after elemination of %%%%%%%
    %%%%%%%%       Weak Interactions in the Real Jacobian          %%%%%% END
    
    PCor = corr(virJ(:),realJ(:));
    SpCor = corr(virJ(:),realJ(:),'type','spearman');
    
    InferenceResult(ii+1,:) = {ii,f,Residual,nzero,tpr1,fpr1,tpr2,fpr2,tpr3,fpr3,PCor,SpCor,LambdaList(ii)};
    BestJacs(:,:,ii) = virJ;
end

OUTPUT.Inference = InferenceResult;
OUTPUT.BestJacs  = BestJacs;

