function Population = sparsecreation(GenomeLength,FitnessFcn,options,MinSparsity)
%SPARSECREATION Creates the initial population for genetic algorithm.
%   POP = SPARSECREATION(NVARS,FITNESSFCN,OPTIONS) Creates the
%   initial population that GA will then evolve into a solution.
%   
%   The function written by JKh (2014/03/19) to generate bitstrings with a
%   user defined minimum sparsity.
%
%   Example:
%     MinSparsity = 0.5;
%     Sparsecreation = @(GenomeLength,FitnessFcn,options)
%     sparsecreation(GenomeLength,FitnessFcn,options,MinSparsity);
%     options = gaoptimset('CreationFcn',Sparsecreation);

if ~strcmpi(options.PopulationType,'bitString')
    error('The PopulationType must be bitString to use this creation function');
end

totalPopulation = sum(options.PopulationSize);
initPopProvided = size(options.InitialPopulation,1);
individualsToCreate = totalPopulation - initPopProvided;


% Initialize Population to be created
Population = zeros(totalPopulation,GenomeLength);
% Use initial population provided already
if initPopProvided > 0
    Population(1:initPopProvided,:) = options.InitialPopulation;
end
% Create remaining population
Population(initPopProvided+1:end,:) = double((1-MinSparsity) > rand(individualsToCreate,GenomeLength));
for i=initPopProvided+1:totalPopulation
    ONgenes = find(Population(i,:));
    NofZeros = GenomeLength-length(ONgenes);
    MinofZeros = ceil(MinSparsity*GenomeLength);
    if NofZeros < MinofZeros
        indx = randperm(length(ONgenes),MinofZeros-NofZeros);
        Population(i,ONgenes(indx)) = 0; % force mutating some 1s to 0s
    end
end

