function mutationChildren = sparsemakermutation(parents,options,GenomeLength,FitnessFcn,state,thisScore,thisPopulation,mutationRate)
%SPARSEMAKERMUTATION Has a tendency to mutate nonzeros to zeros.
%   SPARSEMAKERMUTATION = SPARSEMAKERMUTATION(PARENTS,OPTIONS,GENOMELENGTH,...
%                      FITNESSFCN,STATE,THISSCORE,THISPOPULATION, ...
%                      MUTATIONRATE) Creates the mutated children by
%                      finding the nonzero genes and uniformly and randomly
%                      setting some of them to zero (according to the value of mutationRate).
%
%   The function was written by JKh (2014/03/20) to contribute to
%   generation of sparse children.

if mutationRate > 1 || mutationRate < 0
    error('Mutation rate must be between 0 and 1!')
end
if nargin < 8 || isempty(mutationRate)
    mutationRate = 0.02; % default mutation rate
end
if mutationRate ~= 0 && mutationRate < 0.02
    mutationRate = 0.02;
end

mutationChildren = zeros(length(parents),GenomeLength);
for i=1:length(parents)
    child = thisPopulation(parents(i),:);
    ONgenes = find(child~=0);
    mutationPoints = ONgenes(randperm(length(ONgenes),ceil(mutationRate*length(ONgenes))));
    child(mutationPoints) = 0;
    OFFgenes = find(child==0);
    mutationPoints = OFFgenes(randperm(length(OFFgenes),ceil(0.9*mutationRate*length(ONgenes))));
    child(mutationPoints) = 1;
    mutationChildren(i,:) = child;
end


